@@ -9,21 +9,91 @@ import struct
 from base64 import encodebytes
 from hmac import compare_digest
 from logging import Logger
-from typing import Tuple, Optional, Union, List, Callable
+from typing import Tuple, Optional, Union, List, Callable, Dict
+from urllib.parse import urlparse
 
 from .frame_header import FrameHeader
 
 
-def _open_new_socket(
-    server_hostname: str, server_port: int, logger: Logger
+def _parse_connect_response(sock) -> (Optional[int], str):
+    status = None
+    lines = []
+    while True:
+        line = []
+        while True:
+            c = sock.recv(1)
+            line.append(c)
+            if c == b"\n":
+                break
+        line = b"".join(line).decode("utf-8").strip()
+        if line is None or len(line) == 0:
+            break
+        lines.append(line)
+        if not status:
+            status_line = line.split(" ", 2)
+            status = int(status_line[1])
+    return status, "\n".join(lines)
+
+
+def _establish_new_socket_connection(
+    session_id: str,
+    server_hostname: str,
+    server_port: int,
+    logger: Logger,
+    receive_timeout: float,
+    proxy: Optional[str],
+    proxy_headers: Optional[Dict[str, str]],
+    trace_enabled: bool,
 ) -> Union[ssl.SSLSocket, Socket]:
+    if proxy is not None:
+        parsed_proxy = urlparse(proxy)
+        proxy_host, proxy_port = parsed_proxy.hostname, parsed_proxy.port or 80
+        proxy_addr = socket.getaddrinfo(
+            proxy_host,
+            proxy_port,
+            0,
+            socket.SOCK_STREAM,
+            socket.SOL_TCP,
+        )[0]
+        sock = socket.socket(proxy_addr[0], proxy_addr[1], proxy_addr[2])
+        sock.settimeout(receive_timeout)
+        sock.connect(proxy_addr[4])  # proxy address
+        message = [f"CONNECT {server_hostname}:{server_port} HTTP/1.0"]
+        if proxy_headers is not None:
+            for k, v in proxy_headers.items():
+                message.append(f"{k}: {v}")
+        message.append("")
+        message.append("")
+        req: str = "\r\n".join([line.lstrip() for line in message])
+        if trace_enabled:
+            logger.debug(f"Proxy connect request (session id: {session_id}):\n{req}")
+        sock.send(req.encode("utf-8"))
+        status, text = _parse_connect_response(sock)
+        if status != 200:
+            raise Exception(f"Failed to connect to the proxy (proxy: {proxy})")
+        if trace_enabled:
+            log_message = f"Proxy connect response (session id: {session_id}):\n{text}"
+            logger.debug(log_message)
+
+        sock = ssl.SSLContext(ssl.PROTOCOL_SSLv23).wrap_socket(
+            sock,
+            do_handshake_on_connect=True,
+            suppress_ragged_eofs=True,
+            server_hostname=server_hostname,
+        )
+        return sock
+
     if server_port != 443:
+        addr = socket.getaddrinfo(
+            server_hostname, server_port, 0, socket.SOCK_STREAM, socket.SOL_TCP
+        )[0]
         # only for library testing
         logger.info(
             f"Using non-ssl socket to connect ({server_hostname}:{server_port})"
         )
-        sock = Socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock = Socket(addr[0], addr[1], addr[2])
         sock.settimeout(3)
+        sock.connect((server_hostname, server_port))
         return sock
 
     sock = Socket(type=ssl.SOCK_STREAM)
@@ -33,7 +103,8 @@ def _open_new_socket(
         suppress_ragged_eofs=True,
         server_hostname=server_hostname,
     )
-    sock.settimeout(5)
+    sock.settimeout(receive_timeout)
+    sock.connect((server_hostname, server_port))
     return sock
 
 
@@ -67,7 +138,7 @@ def _parse_handshake_response(sock: ssl.SSLSocket) -> (int, dict, str):
             elements = line.split(":")
             if len(elements) == 2:
                 headers[elements[0].strip().lower()] = elements[1].strip()
-        if line is None or len(line) == 0:
+        if line is None or len(line.strip()) == 0:
             break
         lines.append(line)
     text = "\n".join(lines)
@@ -124,7 +195,8 @@ def _receive_messages(
         )
         received_bytes = sock.recv(size)
         if all_message_trace_enabled:
-            logger.debug(f"Received bytes: {received_bytes}")
+            if len(received_bytes) > 0:
+                logger.debug(f"Received bytes: {received_bytes}")
         return received_bytes
 
     return _fetch_messages(
@@ -301,9 +373,9 @@ def _build_data_frame_for_sending(
     b1 = fin << 7 | rsv1 << 6 | rsv2 << 5 | rsv3 << 4 | opcode
     header: bytes = bytes([b1])
 
-    original_payload_data: bytes = payload.encode("utf-8") if isinstance(
-        payload, str
-    ) else payload
+    original_payload_data: bytes = (
+        payload.encode("utf-8") if isinstance(payload, str) else payload
+    )
     payload_length = len(original_payload_data)
     if payload_length <= 125:
         b2 = masked << 7 | payload_length
